/*
 *  Copyright (C) 2008  Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <udns.h>
#include <unistd.h>
#include <string.h>
#include "tcp_connect.h"
#include "iksemel_extra.h"
#include "xmpp.h"
#include "sasl.h"

int
myhook (void *data, int type, iks *stanza)
{
  if (!iks_strcmp (iks_name (stanza), "stream:features"))
    {
      hc_xmpp_features (data, stanza);
      if (hc_xmpp_is_sasl_supported (data))
        {
          hc_xmpp_sasl_authenticate (data);
        }
    }
  else if (!iks_strcmp (iks_find_attrib (stanza, "xmlns"), HC_XMPP_NS_SASL))
    {
      hc_xmpp_sasl_iterate (data, stanza);
      if (hc_xmpp_status (data) == HC_XMPP_AUTHENTICATED)
        fprintf (stdout, "Authenticated\n");
    }
  else
    {
      fprintf (stderr, "Other: %s\n", iks_string (iks_stack (stanza), stanza));
    }
  return IKS_OK;
}

void
write_stream (hc_xmpp_t *xmpp)
{
  char *buffer = NULL;
  asprintf (&buffer, "<stream:stream xmlns='jabber:client' "
                     "xmlns:stream='http://etherx.jabber.org/streams' "
                     "version='1.0' to='%s'>", hc_xmpp_server (xmpp));
  hc_xmpp_send_buffer (xmpp, buffer, 0);
  free (buffer);
}

void
loop (hc_xmpp_t *xmpp)
{
  while (1)
    hc_xmpp_read_and_parse (xmpp);
}

int
main (int argc, char **argv)
{
  char *server = "jabber-br.org";
  char *user = "pubsub";
  char *password = "pubsub";
  hc_xmpp_t *xmpp;
  dns_init (NULL, 1);
  xmpp = hc_xmpp_new (myhook, server, user, password);
  write_stream (xmpp);
  loop (xmpp);
  return 0;
}
